######################################################################
#
# The Apache Software License, Version 1.1
# 
# Copyright (c) 1999 The Apache Software Foundation.  All rights 
# reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The end-user documentation included with the redistribution,
#    if any, must include the following acknowledgment:  
#       "This product includes software developed by the
#        Apache Software Foundation (http://www.apache.org/)."
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
# 
# 4. The names "Xerces" and "Apache Software Foundation" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written 
#    permission, please contact apache\@apache.org.
# 
# 5. Products derived from this software may not be called "Apache",
#    nor may "Apache" appear in their name, without prior written
#    permission of the Apache Software Foundation.
# 
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# ====================================================================
# 
# This software consists of voluntary contributions made by many
# individuals on behalf of the Apache Software Foundation, and was
# originally based on software copyright (c) 1999, International
# Business Machines, Inc., http://www.ibm.com .  For more information
# on the Apache Software Foundation, please see
# <http://www.apache.org/>.
#
######################################################################
package DOM;
######################################################################

require 5.00560;

use strict;
use utf8;
use Xerces::Xerces;
use Carp;


BEGIN {
	Xerces::init ();
}



######################################################################
# _to_utf8 
#
# A utility that is used throughout this module.  It creates a DOMString2
# object from a DOMString object (which the C++ DOM implementation used to
# encapsulate UTF16 strings.  Then it uses the DOMString2 object to produce
# a UTF8 string for Perl5.  Finally, it destroys the DOMString2 object to
# clean up.
######################################################################


sub _to_utf8 {
	my $dom_string_ref = shift;
	my $ds2 = Xerces::DOMString2_createFromDOMString ($dom_string_ref);
	my $string = $ds2->Xerces::DOMString2_createUTF8 ();
	Xerces::delete_DOMString2 ($ds2);
	return $string;
}




######################################################################
package DOM::Attr;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;


#
# Internal method to create a DOM::Attr that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Attr ($$self);
}


#
# Public methods
#

sub getName {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_Attr_getName () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Attr::getName/;
		croak "$@";
	}

	return $result;
}


sub getValue {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_Attr_getValue () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Attr::getValue/;
		croak "$@";
	}

	return $result;
}


sub setValue {
	my ($self, $value) = @_;

	eval {
		my $domstring = Xerces::DOMString2_createFromUTF8 ($value);
		$$self->Xerces::DOM_Attr_setValue ($domstring);
		Xerces::delete_DOMString2 ($domstring);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Attr::setValue/;
		croak "$@";
	}
}


sub getSpecified {
	my $self = shift;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_Attr_getSpecified ();
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Attr::getSpecified/;
		croak "$@";
	}

	return $result;
}


sub setSpecified {
	my ($self, $specified) = @_;

	eval {
		$$self->Xerces::DOM_Attr_setSpecified ( $specified );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Attr::setSpecified/;
		croak "$@";
	}
}



######################################################################
package DOM::CDATASection;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Text);


#
# Internal method to create a DOM::CDATASection that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_CDATASection ($$self);
}



######################################################################
package DOM::CharacterData;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;


#
# Internal method to create a DOM::CharacterData that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_CharacterData ($$self);
}


#
# Public methods
#

sub getData {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8( $$self->Xerces::DOM_CharacterData_getData () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::getData/;
		croak "$@";
	}

	return $result;
}


sub setData {
	my ($self, $data) = @_;

	eval {
		$$self->Xerces::DOM_CharacterData_setData ( DOM::to_domstring($data) );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::setData/;
		croak "$@";
	}
}


sub getLength {
	my $self = shift;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_CharacterData_getLength ();
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::getLength/;
		croak "$@";
	}

	return $result;
}


sub substringData {
	my ($self, $offset, $count) = @_;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_CharacterData_substringData ($offset, $count) );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::substringData/;
		croak "$@";
	}

	return $result;
}


sub appendData {
	my ($self, $data) = @_;

	eval {
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);
		$$self->Xerces::DOM_CharacterData_appendData ($_data);
		Xerces::delete_DOMString2 ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::appendData/;
		croak "$@";
	}
}


sub insertData {
	my ($self, $offset, $data) = @_;

	eval {
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);		
		$$self->Xerces::DOM_CharacterData_insertData ($offset, $_data);
		Xerces::delete_DOMString2 ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::insertData/;
		croak "$@";
	}
}


sub deleteData 
{
	my ($self, $offset, $count) = @_;

	eval {
		$$self->Xerces::DOM_CharacterData_deleteData ($offset, $count);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::deleteData/;
		croak "$@";
	}
}


sub replaceData {
	my ($self, $offset, $count, $data) = @_;

	eval {
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);		
		$$self->Xerces::DOM_CharacterData_replaceData ($offset, $count, $_data);
		Xerces::delete_DOMString2 ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::CharacterData::replaceData/;
		croak "$@";
	}
}



######################################################################
package DOM::Comment;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::CharacterData);


#
# Internal method to create a DOM::Comment that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Comment ($$self);
}


######################################################################
package DOM::Document;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;


#
# Internal method to create a DOM::Document that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Document ($$self);
}


#
# Class method to create a document
#


sub createDocument {
	my $result;

	eval {
		$result = DOM::Document->_wrap ( Xerces::DOM_Document_createDocument () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createDocument/;
		croak "$@";
	}

	return $result;
}



#
# Public methods
#

sub createEntity {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);		
		$result = DOM::Entity->_wrap ($$self->Xerces::DOM_Document_createEntity ($_name));
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createEntity/;
		croak "$@";
	}

	return $result;
}


sub createElement {
	my ($self, $tag_name) = @_;
	my $result;

	eval {
		my $_tag_name = Xerces::DOMString2_createFromUTF8 ($tag_name);		
		$result = DOM::Element->_wrap ($$self->Xerces::DOM_Document_createElement ($_tag_name));
		Xerces::delete_DOMString2 ($_tag_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createElement/;
		croak "$@";
	}

	return $result;
}


sub createDocumentFragment {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::DocumentFragment->_wrap ($$self->Xerces::DOM_Document_createDocumentFragment ());
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createDocumentFragment/;
		croak "$@";
	}

	return $result;
}


sub createTextNode {
	my ($self, $data) = @_;
	my $result;

	eval {
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);		
		$result = DOM::Text->_wrap ($$self->Xerces::DOM_Document_createTextNode ($_data));
		Xerces::delete_DOMString2 ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createTextNode/;
		croak "$@";
	}

	return $result;
}


sub createComment {
	my ($self, $data) = @_;
	my $result;

	eval {
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);		
		$result = DOM::Comment->_wrap ($$self->Xerces::DOM_Document_createComment ($_data));
		Xerces::delete_DOMString2 ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createComment/;
		croak "$@";
	}

	return $result;
}


sub createCDATASection {
	my ($self, $data) = @_;
	my $result;

	eval {
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);		
		$result = DOM::CDATASection->_wrap ($$self->Xerces::DOM_Document_createCDATASection ($_data));
		Xerces::delete_DOMString2 ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createCDATASection/;
		croak "$@";
	}

	return $result;
}


sub createDocumentType {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);		
		$result = DOM::DocumentType->_wrap ($$self->Xerces::DOM_Document_createDocumentType ($_name));
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createDocumentType/;
		croak "$@";
	}

	return $result;
}


sub createNotation {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);		
		$result = DOM::Notation->_wrap ($$self->Xerces::DOM_Document_createNotation ($_name));
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createNotation/;
		croak "$@";
	}

	return $result;
}


sub createProcessingInstruction {
	my ($self, $target, $data) = @_;
	my $result;

	eval {
		my $_target = Xerces::DOMString2_createFromUTF8 ($target);
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);
		my $inst = $$self->Xerces::DOM_Document_createProcessingInstruction ($_target, $_data);
		$result = DOM::ProcessingInstruction->_wrap ($inst);
		Xerces::delete_DOMString2 ($_target);
		Xerces::delete_DOMString2 ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createProcessingInstruction/;
		croak "$@";
	}

	return $result;
}

sub createAttribute {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);		
		$result = DOM::Attr->_wrap($$self->Xerces::DOM_Document_createAttribute($_name));
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createAttribute/;
		croak "$@";
	}

	return $result;
}


sub createEntityReference {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);		
		$result = DOM::EntityReference->_wrap($$self->Xerces::DOM_Document_createEntityReference ($_name));
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createEntityReference/;
		croak "$@";
	}

	return $result;
}


sub getDoctype {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::DocumentType->_wrap ($$self->Xerces::DOM_Document_getDoctype() );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::createDoctype/;
		croak "$@";
	}

	return $result;
}


sub getImplementation {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::DOMImplementation->_wrap ( $$self->Xerces::DOM_Document_getImplementation () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::getImplementation/;
		croak "$@";
	}

	return $result;
}


sub getDocumentElement {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::Element->_wrap ( $$self->Xerces::DOM_Document_getDocumentElement () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::getDocumentElement/;
		croak "$@";
	}

	return $result;
}


sub getElementsByTagName {
	my ($self, $tag_name) = @_;
	my $result;

	eval {
		my $_tag_name = Xerces::DOMString2_createFromUTF8 ($tag_name);		
		$result = DOM::NodeList->_wrap ($$self->Xerces::DOM_Document_getElementsByTagName ($_tag_name));
		Xerces::delete_DOMString2 ($_tag_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::getElementsByTagName/;
		croak "$@";
	}

	return $result;
}


sub importNode {
	my ($self, $source_node, $deep_bool) = @_;
	my $result;

	eval {
		$result = DOM::NodeList->_wrap ( $$self->Xerces::DOM_Document_getElementsByTagName ($$source_node, $deep_bool) );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Document::importNode/;
		croak "$@";
	}

	return $result;
}



######################################################################
package DOM::DocumentFragment;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);


#
# Internal method to create a DOM::DocumentFragment that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_DocumentFragment ($$self);
}


######################################################################
package DOM::DocumentType;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;


#
# Internal method to create a DOM::DocumentType that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_DocumentType ($$self);
}



#
# Public methods
#

sub getName {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_DocumentType_getName () );
	};
		
	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::DocumentType::getName/;
		croak "$@";
	}

	return $result;
}


sub getEntities {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::NamedNodeMap->_wrap ( $$self->Xerces::DOM_DocumentType_getEntities () );
	};
		
	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::DocumentType::getEntities/;
		croak "$@";
	}

	return $result;
}


sub getNotations {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::NamedNodeMap->_wrap ( $$self->Xerces::DOM_DocumentType_getNotations () );
	};
		
	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::DocumentType::getNotations/;
		croak "$@";
	}

	return $result;
}


######################################################################
package DOM::DOMImplementation;
######################################################################

use Carp;


#
# Internal method to create a DOM::DOMImplementation that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_DOMImplementation ($$self);
}



#
# Public methods
#

sub getImplementation {
	my $result;
	
	eval {	
		$result = DOM::DOMImplementation->_wrap ( Xerces::DOM_DOMImplementation_getImplementation () );
	};
		
	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::DOMImplementation::getImplementation/;
		croak "$@";
	}

	return $result;
}


sub hasFeature {
	my ($self, $feature, $version) = @_;
	my $result;

	eval {
		my $_feature = Xerces::DOMString2_createFromUTF8 ($feature);
		my $_version = Xerces::DOMString2_createFromUTF8 ($version);
		$result = $$self->Xerces::DOM_DOMImplementation_hasFeature ($_feature, $_version);
		Xerces::delete_DOMString2 ($_feature);
		Xerces::delete_DOMString2 ($_version);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::DOMImplementation::hasFeature/;
		croak "$@";
	}

	return $result;
}


######################################################################
package DOM::Element;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;


#
# Internal method to create a DOM::Element that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Element ($$self);
}



#
# Public methods
#

sub getTagName {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ($$self->Xerces::DOM_Element_getTagName ());
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::getTagName/;
		croak "$@";
	}

	return $result;
}


sub getAttribute {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);
		$result = DOM::_to_utf8 ($$self->Xerces::DOM_Element_getAttribute ($_name));
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::getAttribute/;
		croak "$@";
	}

	return $result;
}


sub getAttributeNode {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);
		my $attribute_node = $$self->Xerces::DOM_Element_getAttributeNode ($_name);
		$result = DOM::Attr->_wrap ($attribute_node);
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::getAttributeNode/;
		croak "$@";
	}

	return $result;
}


sub getElementsByTagName {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);
		my $elements = $$self->Xerces::DOM_Element_getElementsByTagName ($_name);
		$result = DOM::NodeList->_wrap ($elements);
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::getElementsByTagName/;
		croak "$@";
	}

	return $result;
}


sub setAttribute {
	my ($self, $name, $value) = @_;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);
		my $_value = Xerces::DOMString2_createFromUTF8 ($value);
		$$self->Xerces::DOM_Element_setAttribute ($_name, $_value);
		Xerces::delete_DOMString2 ($_name);
		Xerces::delete_DOMString2 ($_value);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::setAttribute/;
		croak "$@";
	}
}


sub setAttributeNode {
	my ($self, $node) = @_;

	eval {
		$$self->Xerces::DOM_Element_setAttributeNode ($$node);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::getAttributeNode/;
		croak "$@";
	}
}


sub removeAttribute {
	my ($self, $name) = @_;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);
		$$self->Xerces::DOM_Element_removeAttribute ($_name);
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::removeAttribute/;
		croak "$@";
	}
}


sub removeAttributeNode {
	my ($self, $old_attr) = @_;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_Element_removeAttributeNode ($$old_attr);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::removeAttributeNode/;
		croak "$@";
	}

	return $result;
}


sub normalize {
	my $self = shift;

	eval {
		$$self->Xerces::DOM_Element_normalize ();
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Element::normalize/;
		croak "$@";
	}
}



######################################################################
package DOM::Entity;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;


#
# Internal method to create a DOM::Entity that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Entity ($$self);
}



#
# Public methods
#

sub getPublicId {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_Entity_getPublicId () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Entity::getPublicId/;
		croak "$@";
	}

	return $result;
}


sub getSystemId {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_Entity_getSystemId () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Entity::getSystemId/;
		croak "$@";
	}

	return $result;
}


sub getNotationName {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_Entity_getNotationName () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Entity::getNotationName/;
		croak "$@";
	}

	return $result;
}


sub setPublicId {
	my ($self, $id) = @_;
	
	eval {
		my $_id = Xerces::DOMString2_createFromUTF8 ($id);
		$$self->Xerces::DOM_Entity_setPublicId ($_id);
		Xerces::delete_DOMString2 ($_id);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Entity::setPublicId/;
		croak "$@";
	}
}


sub setSystemId {
	my ($self, $id) = @_;

	eval {
		my $_id = Xerces::DOMString2_createFromUTF8 ($id);
		$$self->Xerces::DOM_Entity_setSystemId ($_id);
		Xerces::delete_DOMString2 ($_id);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Entity::setSystemId/;
		croak "$@";
	}
}


sub setNotationName {
	my ($self, $name) = @_;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);
		$$self->Xerces::DOM_Entity_setNotationName ($_name);
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Entity::setNotationName/;
		croak "$@";
	}
} 



######################################################################
package DOM::EntityReference;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);


#
# Internal method to create a DOM::EntityReference that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}

#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_EntityReference ($$self);
}



######################################################################
package DOM::NamedNodeMap;
######################################################################

use Carp;


#
# Internal method to create a DOM::NamedNodeMap that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_NamedNodeMap ($$self);
}



#
# Public methods
#

sub setNamedItem {
	my ($self, $arg) = @_;

	eval {
		DOM::Node->_wrap ( $$self->Xerces::DOM_NamedNodeMap_setNamedItem ($$arg) );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::NamedNodeMap::setNamedItem/;
		croak "$@";
	}
}


sub item {
	my ($self, $index) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_NamedNodeMap_item ($index) );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::NamedNodeMap::item/;
		croak "$@";
	}

	return $result;
}


sub getNamedItem {
	my ($self, $name) = @_;
	my $result;

	eval {
		my $_name = Xerces::DOMString2_createFromUTF8 ($name);
		$result = DOM::Node->_wrap ($$self->Xerces::DOM_NamedNodeMap_getNamedItem($_name));
		Xerces::delete_DOMString2 ($_name);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::NamedNodeMap::getNamedItem/;
		croak "$@";
	}

	return $result;
}


sub getLength {
	my $self = shift;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_NamedNodeMap_getLength ();
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::NamedNodeMap::getLength/;
		croak "$@";
	}

	return $result;
}


sub removeNamedItem {
	my ($self, $name) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_NamedNodeMap_removeNamedItem ( DOM::to_domstring($name) ) );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::NamedNodeMap::removeNamedItem/;
		croak "$@";
	}

	return $result;
}



######################################################################
package DOM::Node;
######################################################################

use vars qw (@ISA @EXPORT);
@ISA = qw (Exporter);

use Carp;


    
#
# Enumeration
#

@EXPORT = qw (
	     ELEMENT_NODE
	     ATTRIBUTE_NODE
	     TEXT_NODE
	     CDATA_SECTION_NODE
	     ENTITY_REFERENCE_NODE
	     ENTITY_NODE
	     PROCESSING_INSTRUCTION_NODE
	     COMMENT_NODE
	     DOCUMENT_NODE
	     DOCUMENT_TYPE_NODE
	     DOCUMENT_FRAGMENT_NODE
	     NOTATION_NODE
	     );


sub ELEMENT_NODE () 				{1;}
sub ATTRIBUTE_NODE ()				{2;}
sub TEXT_NODE ()					{3;}
sub CDATA_SECTION_NODE ()			{4;}
sub ENTITY_REFERENCE_NODE ()		{5;}
sub ENTITY_NODE	()					{6;}
sub PROCESSING_INSTRUCTION_NODE ()	{7;}
sub COMMENT_NODE ()					{8;}
sub DOCUMENT_NODE ()				{9;}
sub DOCUMENT_TYPE_NODE ()			{10;}
sub DOCUMENT_FRAGMENT_NODE ()		{11;}
sub NOTATION_NODE					{12;}


#
# Internal method to create a DOM::Node that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Node ($$self);
}



#
# Public instance methods
#

sub getNodeName {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ($$self->Xerces::DOM_Node_getNodeName ());
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getNodeName/;
		croak "$@";
	}

	return $result;
}


sub getNodeValue {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ($$self->Xerces::DOM_Node_getNodeValue ());;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getNodeValue/;
		croak "$@";
	}

	return $result;
}


sub getNodeType {
	my $self = shift;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_Node_getNodeType ();;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getNodeType/;
		croak "$@";
	}

	return $result;
}


sub getParentNode {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_getParentNode () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getParentNode/;
		croak "$@";
	}

	return $result;
}


sub getChildNodes {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::NodeList->_wrap ( $$self->Xerces::DOM_Node_getChildNodes () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getChildNodes/;
		croak "$@";
	}

	return $result;
}


sub getFirstChild {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_getFirstChild () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getFirstChild/;
		croak "$@";
	}

	return $result;
}


sub getLastChild {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_getLastChild () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getLastChild/;
		croak "$@";
	}

	return $result;
}


sub getPreviousSibling {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_getPreviousSibling () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getPreviousSibling/;
		croak "$@";
	}

	return $result;
}


sub getNextSibling {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_getNextSibling () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getNextSibling/;
		croak "$@";
	}

	return $result;
}


sub getAttributes {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::NamedNodeMap->_wrap ( $$self->Xerces::DOM_Node_getAttributes () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getAttributes/;
		croak "$@";
	}

	return $result;
}


sub getOwnerDocument {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::Document->_wrap ( $$self->Xerces::DOM_Node_getOwnerDocument () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::getOwnerDocument/;
		croak "$@";
	}

	return $result;
}


sub cloneNode {
	my ($self, $deep) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_getCloneNode ($deep) );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::cloneNode/;
		croak "$@";
	}

	return $result;
}


sub insertBefore {
	my ($self, $new_child, $ref_child) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_insertBefore ($$new_child, $$ref_child) );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::insertBefore/;
		croak "$@";
	}

	return $result;
}


sub replaceChild {
	my ($self, $new_child, $old_child) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_replaceChild ($$new_child, $$old_child) );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::replaceChild/;
		croak "$@";
	}

	return $result;
}


sub removeChild {
	my ($self, $old_child) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_removeChild ($$old_child) );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::removeChild/;
		croak "$@";
	}

	return $result;
}


sub appendChild {
	my ($self, $new_child) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_Node_appendChild ($$new_child) );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::appendChild/;
		croak "$@";
	}

	return $result;
}


sub hasChildNodes {
	my $self = shift;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_Node_hasChildNodes ();;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::hasChildNodes/;
		croak "$@";
	}

	return $result;
}


sub isNull {
	my $self = shift;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_Node_isNull ();
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::isNull/;
		croak "$@";
	}

	return $result;
}


sub setNodeValue {
	my ($self, $node_value) = @_;

	eval {
		my $_node_value = Xerces::DOMString2_createFromUTF8 ($node_value);
		$$self->Xerces::DOM_Node_setNodeValue ($_node_value);
		Xerces::delete_DOMString2 ($_node_value);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Node::setNodeValue/;
		croak "$@";
	}
}



######################################################################
package DOM::NodeList;
######################################################################

use Carp;


#
# Internal method to create a DOM::NodeList that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_NodeList ($$self);
}



#
# Public methods
#

sub getLength {
	my $self = shift;
	my $result;

	eval {
		$result = $$self->Xerces::DOM_NodeList_getLength ();
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::NodeList::getLength/;
		croak "$@";
	}

	return $result;
}


sub item {
	my ($self, $item) = @_;
	my $result;

	eval {
		$result = DOM::Node->_wrap ( $$self->Xerces::DOM_NodeList_item ($item) );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::NodeList::item/;
		croak "$@";
	}

	return $result;
}



######################################################################
package DOM::Notation;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;



#
# Internal method to create a DOM::Notation that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Notation ($$self);
}



#
# Public methods
#

sub getPublicId {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_Notation_getPublicId () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Notation::getPublicId/;
		croak "$@";
	}

	return $result;
}


sub getSystemId {
	my $self = shift;
	my $result;

	eval {
		$result = 	DOM::_to_utf8 ( $$self->Xerces::DOM_Notation_getSystemId () );;
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Notation::getSystemId/;
		croak "$@";
	}

	return $result;
}


sub setPublicId {
	my ($self, $id) = @_;

	eval {
		my $_id = Xerces::DOMString2_createFromUTF8 ($id);
		$$self->Xerces::DOM_Notation_setPublicId ($_id);
		Xerces::delete_DOMString2 ($_id);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Notation::setPublicId/;
		croak "$@";
	}
}


sub setSystemId {
	my ($self, $id) = @_;

	eval {
		my $_id = Xerces::DOMString2_createFromUTF8 ($id);
		$$self->Xerces::DOM_Notation_setSystemId ($_id);
		Xerces::delete_DOMString2 ($_id);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Notation::setSystemId/;
		croak "$@";
	}
}


######################################################################
package DOM::ProcessingInstruction;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::Node);

use Carp;


#
# Internal method to create a DOM::ProcessingInstruction that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_ProcessingInstruction ($$self);
}



#
# Public methods
#

sub getTargetData {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_ProcessingInstruction_getTargetData () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::ProcessingInstruction::getTargetData/;
		croak "$@";
	}

	return $result;
}


sub getData {
	my $self = shift;
	my $result;

	eval {
		$result = DOM::_to_utf8 ( $$self->Xerces::DOM_ProcessingInstruction_getData () );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::ProcessingInstruction::getData/;
		croak "$@";
	}

	return $result;
}


sub setData {
	my ($self, $data) = @_;

	eval {
		my $_data = Xerces::DOMString2_createFromUTF8 ($data);
		$$self->Xerces::DOM_ProcessingInstruction_setData ($_data);
		Xerces::delete_DOMString ($_data);
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::ProcessingInstruction::setData/;
		croak "$@";
	}
}



######################################################################
package DOM::Text;
######################################################################

use vars qw (@ISA);
@ISA = qw (DOM::CharacterData);

use Carp;


#
# Internal method to create a DOM::Text that wraps the 
# corresponding SWIG reference.  INTERNAL USE ONLY!!
#

sub _wrap {
	my ($class, $swig_ref) = @_;
	return bless \$swig_ref, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOM_Text ($$self);
}


#
# Public methods
#

sub splitText {
	my ($self, $offset) = @_;
	my $result;

	eval {
		$result = DOM::Text->_wrap ( $$self->Xerces::DOM_Text_splitText ($offset) );
	};

	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Text::splitText/;
		croak "$@";
	}

	return $result;
}



######################################################################
package DOM::Parser;
######################################################################
 
use Carp;


#
# Public methods
#

sub new {
	my ($class) = @_;
	my $rep = Xerces::new_DOMParser2 ();
	return bless \$rep, $class;
}


#
# Called when no longer referenced.  Deletes underlying
# SWIG reference.
#

sub DESTROY {
	my $self = shift;
	Xerces::delete_DOMParser2 ($$self);
}


sub setDoValidation {
	my ($self, $doValidation) = @_;

	eval {
		$$self->Xerces::DOMParser2_setDoValidation ($doValidation);
	}; 
		
	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Parser::setDoValidation/;
		croak "$@";
	}
}


sub parse {
	my ($self, $uri) = @_;

	eval {
		$$self->Xerces::DOMParser2_parse ($uri);
	}; 
		
	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Parser::parse/;
		croak "$@";
	}
}


sub getDocument {
	my $self = shift;
	my $result;

	eval {
		my $document = $$self->Xerces::DOMParser2_getDocument ();
		$result = DOM::Document->_wrap ($document);
	};
	 
	if ( $@ ) {
		$@ =~ s/(.*) at .+/$1 in DOM::Parser::getDocument/;
		croak "$@";
	}

	return $result;
}


=head1 NAME

Xerces::DOM -  A Perl module for parsing XML documents with the W3C DOM API.


=head1 SYNOPSIS


	# Here is a simple example script to count the nodes in an 
	# XML document.  The document file name is passed on the 
	# command line.

	use Xerces::DOM;

	my $parser = new DOM::Parser();
	$parser->parse($ARGV[0]);
	my $document = $parser->getDocument();
	my $element_count = $document->getElementsByTagName("*")->getLength();
	print "$file: ($element_count elems)\n";


=head1 DESCRIPTION

This module provides ways to read in and parse XML files using the W3C
standard DOM APIs.  It is built on the industry standard Apache Xerces
C++ Parser (formerly IBM XML4C).  It supports validation for 
well-formedness and optionally for correctness against a corresponding
DTD file.

DOM objects are exposed as Perl5 objects with APIs modeled after the W3C 
standard Java language binding (the only official language binding at the
time of developing this).  The Parser has full 
Unicode support and can read various standard encodings.  Strings are 
represented internally as UTF16 and are accessed through Perl APIs as 
UTF8 strings if you are using this module with Perl5 experimental build 
5.00560 with Unicode support.  Use the UTF8 pragma to enable UTF8 strings 
in your scripts, e.g.

	use utf8;

=head1 CLASSES

=head2 DOM::Attr

=over 4

=item Base Class: 

DOM::Node

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getName ()

Returns a string.

=item getSpecified ()

Returns true (1) or false (0).

=item getValue ()

Returns a string.

=item setValue (string value)

=item setSpecified (int specified)

=back

=back

=head2 DOM::CDATASection

=over 4

=item Base Class: 

DOM::Text

=item Class Methods:

None

=item Instance Methods:

None

=back

=head2 DOM::CharacterData

=over 4

=item Base Class:

DOM::Node

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getData ()

Returns a string.

=item getLength ()

Returns an integer.

=item substringData (int offset, int count)

Returns a string.  May throw a DOM exception.

=item appendData (string data)

May throw a DOM exception.

=item insertData (int offset, string data)

May throw a DOM exception.

=item deleteData (int offset, int count)

May throw a DOM exception.

=item replaceData (int offset, int count, string arg)

May throw a DOM exception.

=item setData (string data)

May throw a DOM exception.

=back

=back

=head2 DOM::Comment

=over 4

=item Base Class: 

DOM::CharacterData

=item Class Methods:

None

=item Instance Methods:

None

=back

=head2 DOM::Document

=over 4

=item Base Class:

DOM::Node

=item Class Methods:

=over 4

=item createDocument ();

Returns a new DOM::Document.

=back

=item Instance Methods:

=over 4

=item createEntity (string name);

Returns a new DOM::Entity.

=item createElement (string tag_name);

Returns a new DOM::Element.

May throw an exception.

=item createDocumentFragment ();

Returns a new DOM::DocumentFragment.

=item createTextNode (string data);

Returns a new DOM::Text.

=item createComment (string data);

Returns a new DOM::Comment.

=item createCDATASection (string data)

Returns a new DOM::CDATASection.

=item createDocumentType (string name)

Returns a new DOM::DocumentType.

=item createNotation (string name)

Returns a new DOM::Notation.

=item createProcessingInstruction (string target, string data)

Returns a new DOM::ProcessingInstruction.

May throw a DOM exception.

=item createAttribute (string name)

Returns a new DOM::Attr.

May throw a DOM exception.

=item createEntityReference (string name)

Returns a new DOM::EntityRefernce.

May throw a DOM exception.

=item getDoctype ()

Returns a DOM::DocumentType.

=item getImplementation ()

Returns a DOM::DOMImplementation.

=item getDocumentElement()

Returns a DOM::Element.

=item getElementsByTagName(string tag_name)

Returns a DOM::NodeList.

=item importNode (DOM::Node source, bool deep)

Returns a DOM::Node.

=back

=back

=head2 DOM::DocumentFragment

=over 4

=item Base Class: 

DOM::Node

=item Class Methods:

None

=item Instance Methods:

None

=back

=head2 DOM::DocumentType

=over 4

=item Base Class:

DOM::Node

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getName ()

Returns a string.

=item getEntities ()

Returns a DOM::NamedNodeMap.

=item getNotations ()

Returns a DOM::NamedNodeMap.

=back

=back

=head2 DOM::DOMImplementation

=over 4

=item Base Class:

None

=item Class Methods:

=over 4

=item getImplementation ();

Returns a DOM::DOMImplementation.

=back

=item Instance Methods:

=over 4

=item hasFeature (string feature,  string version)

Returns true (1) or false (0).

=back

=back

=head2 DOM::Element

=over 4

=item Base Class:

DOM::Node

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getTagName ()

Returns a string.

=item getAttribute (string name)

Returns a string.

=item getAttributeNode (string name)

Returns a DOM::Attr.

=item getElementsByTagName (string name)

Returns a DOM::NodeList.

=item setAttribute (string name, strnig value)

May throw a DOM exception.

=item setAttributeNode (DOM::Attr newAttr)

Returns a DOM::Attr.

May throw a DOM exception.

=item removeAttributeNode (DOM::Attr oldAttr)

Returns a DOM::Attr.

May throw a DOM exception.

=item normalize ()

=item removeAttribute (string name)

May throw a DOM exception.

=back

=back

=head2 DOM::Entity

=over 4

=item Base Class:

DOM::Node

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getPublicId ()

Returns a string.

=item getSystemId ()

Returns a string.

=item getNotationName ()

Returns a string.

=item setNotationName (string name)

=item setPublicId (string id)

=item setSystemId (string id)

=back

=back

=head2 DOM::EntityReference

=over 4

=item Base Class: 

DOM::Node

=item Class Methods:

None

=item Instance Methods:

None

=back

=head2 DOM::NamedNodeMap

=over 4

=item Base Class:

None

=item Class Methods:

None

=item Instance Methods:

=over 4

=item setNamedItem (DOM::Node arg)

Returns a DOM::Node.

May throw a DOM exception.

=item item (int index)

Returns an integer.

=item getNamedItem (string name)

Returns a DOM::Node.

=item getLength ()

Returns an integer.

=item removeNamedItem (string name)

Removes a DOM::Node.

May throw a DOM exception.

=back

=back

=head2 DOM::Node

=over 4

=item Base Class:

None

=item Class Constants:

=over 4

=item ELEMENT_NODE = 1

=item ATTRIBUTE_NODE = 2

=item TEXT_NODE = 3

=item CDATA_SECTION_NODE = 4

=item ENTITY_REFERENCE_NODE = 5

=item ENTITY_NODE = 6

=item PROCESSING_INSTRUCTION_NODE = 7

=item COMMENT_NODE = 8

=item DOCUMENT_NODE = 9

=item DOCUMENT_TYPE_NODE = 10

=item DOCUMENT_FRAGMENT_NODE = 11

=item NOTATION_NODE = 12

=back

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getNodeName ()

Returns a string.

=item getNodeValue ()

Returns a string.

May throw a DOM exception.

=item getNodeType ()

Returns an integer.

=item getParentNode ()

Returns a DOM::Node.

=item getChildNodes ()

Returns a DOM::NodeList.

=item getFirstChild ()

Returns a DOM::Node.

=item getLastChild ()

Returns a DOM::Node.

=item getPreviousSibling ()

Returns a DOM::Node.

=item getNextSibling ()

Returns a DOM::Node.

=item getAttributes ()

Returns a DOM::NamedNodeMap.

=item getOwnerDocument ()

Returns a DOM::Document.

=item cloneNode (boolean deep)

Returns a DOM::Node.

=item insertBefore (DOM::Node new_child, DOM::Node ref_child)

Returns a DOM::Node.

May throw a DOM exception.

=item replaceChild (DOM::Node new_child, DOM::Node old_child)

Returns a DOM::Node.

May throw a DOM exception.

=item removeChild (DOM::Node old_child)

Returns a DOM::Node.

May throw a DOM exception.

=item appendChild (DOM::Node new_child)

Returns a DOM::Node.

May throw a DOM exception.

=item hasChildNodes ()

Returns true (1) or false (0).

=item boolean isNull ()

Returns true (1) or false (0).

=item setNodeValue (string node_value)

May throw a DOM exception.

=back

=back

=head2 DOM::NodeList

=over 4

=item Base Class:

None

=item Class Methods:

None

=item Instance Methods:

=over 4

=item item (int index)

Returns a DOM::Node.

=item getLength ()

Returns an integer.

=back

=back

=head2 DOM::Notation

=over 4

=item Base Class:

DOM::Node

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getPublicId ()

Returns a string.

=item getSystemId ()

Returns a string.

=item setPublicId (string id)

=item setSystemId (string id)

=back

=back
 
=head2 DOM::ProcessingInstruction

=over 4

=item Base Class: 

DOM::Node

=item Class Methods:

None

=item Instance Methods:

=over 4

=item getTarget ()

Returns a string.

=item getData ()

Returns a string.

=item setData (string data)

May throw a DOM exception.

=back

=back

=head2 DOM::Text

=over 4

=item Base Class: 

DOM::CharacterData

=item Class Methods:

None

=item Instance Methods:

=over 4

=item splitText (int offset)

Returns a DOM::Text.

May throw a DOM exception.

=back

=back


=head1 OTHER DETAILS

=head2 Null References

Xerces DOM methods never return NULL object references, i.e. you never have to call defined() to check them.  However, in certain error situations the objects returned represent NULL objects.  You can test for this condition by calling the DOM::Node->isNull() method.


=head2 Exception Handling

Exceptions are caught from the underlying C++ parser and are rethrown
using Croak calls (see Carp).  All exceptions are thrown as strings 
messages which include the following standard DOM identifiers.  

=over 4

=item "INDEX_SIZE_ERR"

=item "DOMSTRING_SIZE_ERR"

=item "HIERARCHY_REQUEST_ERR"

=item "WRONG_DOCUMENT_ERR"

=item "INVALID_CHARACTER_ERR"

=item "NO_DATA_ALLOWED_ERR"

=item "NO_MODIFICATION_ALLOWED_ERR"

=item "NOT_FOUND_ERR"

=item "NOT_SUPPORTED_ERR"

=item "INUSE_ATTRIBUTE_ERR"

=item "Unknown exception"

=back

Exception strings also include the DOM class and method from which the 
exception was thrown, and the line number of your calling script, e.g.

	INDEX_SIZE_ERR in DOM::Text::splitText
		at YourScript.pl line 11

You can let exceptions float to the top as error messages or you can catch 
and handle exceptions as follows:

	eval {
		# your code goes here
	};

	if ( $@ ) {
		# exception caught
		# exception message in $@ as a string
	}

You can get complete stack crawls in exception messages by invoking the
Perl interpreter with the "-MCarp=verbose" option, e.g.

	perl -MCarp=verbose script.pl

=head2 Unicode Strings

The Xerces Perl DOM is built on the very powerful Unicode-enabled 
Xerces C++ XML Parser.  The C++ parser can read various Unicode
encodings and it represents strings internally in UTF16.  But you
pass strings in and out of methods as standard Perl strings.  If
you plan to take advantage of Unicode, make sure to do two things:

=over 4

=item 1. Use the experimental Perl5 build 5.00560 with Unicode support.

=item 2. Use the "Use UTF8" pragma in your scripts.

=back

When you do these things, all strings that you pass in and out will
be UTF8 strings and the Xerces Perl DOM will automatically convert
these to UTF16 internally.

=head2 Object Creation

You create DOM::Parser and DOM::Document objects using their respective
static new methods.  All other DOM objects are created by calling the
corresponding static factory methods on DOM::Document, or are returned
from various other methods on DOM objects.  This is standard DOM stuff,
but I thought I'd mention it so you don't pull your hair out looking
for new methods on the other objects.

=head2 Object Destruction

Basically, you need not worry about this.  All memory management of 
underlying C++ and Perl memory is managed for you.  When you no longer
have any references to a Xerces DOM object, its destructor is called 
automatically and it cleans up all associated memory.  

=head1 AUTHORS

Tom Watson <F<rtwatson@us.ibm.com>> wrote version 1.0 and submitted to
the XML Apache project <F<http://xml.apache.org>>, where you can contribute 
to future versions and where the corresponding C++ and Java compilers are
also developed as OpenSource projects.




