
/*
 * The Apache Software License, Version 1.1
 * 
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 * 
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache\@apache.org.
 * 
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation, and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.ibm.com .  For more information
 * on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */


#include "DOMString2.hpp"
#include <unicode.h>
#include <convert.h>
#include <ustring.h>
#include <string.h>



DOMString2		
DOMString2::createFromDOMString (const DOMString& source)
{
	return DOMString2 (source);
}
	

DOMString2		
DOMString2::createFromUTF8 (char* utf8)
{
	return DOMString2 (utf8);
}
	



DOMString2::DOMString2 (const DOMString& source) :
	DOMString (source),
	fCharBuffer (0)
{
}




DOMString2::DOMString2 (char* utf8) :
	fCharBuffer (0)
{
	if (utf8 == 0) return;

	// Create converter
	UErrorCode err = ZERO_ERROR;
	UConverter* converter = ucnv_open ("utf8", &err);

	UChar* dstString = 0;
	int32_t srcLen = (int32_t)strlen(utf8);
	int32_t dstLen = 0;
        
	if (srcLen == 0) return;

	// Compute dstLen
	err = ZERO_ERROR;
	dstLen = ucnv_toUChars (converter, dstString, dstLen, utf8, srcLen, &err);
            
	// Allocate dstString
	dstString = new UChar [dstLen+1];

	// Convert
	err = ZERO_ERROR;
	ucnv_toUChars (converter, dstString, dstLen+1, utf8, srcLen, &err);

	// Close converter
	err = ZERO_ERROR;
	ucnv_close (converter);
    
	appendData (DOMString(dstString));
	delete [] dstString;

}


DOMString2::~DOMString2 ()
{
	delete [] fCharBuffer;
}





char*
DOMString2::createUTF8 ()
{
	UChar* srcString = rawBuffer ();

	if ( fCharBuffer != 0 ) {
		delete [] fCharBuffer;
		fCharBuffer = 0;
	}

	if (srcString == 0) return 0;

	// Create converter

	UErrorCode err = ZERO_ERROR;
	UConverter* converter = ucnv_open ("utf8", &err);

	int32_t dstLen = 0;
        
	// Compute dstLen
	err = ZERO_ERROR;
	dstLen = ucnv_fromUChars (converter, fCharBuffer, dstLen, srcString, &err);
            
	// Allocate dstString
	fCharBuffer = new char [dstLen+1];

	// Convert
	err = ZERO_ERROR;
	ucnv_fromUChars (converter, fCharBuffer, dstLen+1, srcString, &err);
	fCharBuffer[dstLen] = 0;

	// Close converter
	err = ZERO_ERROR;
	ucnv_close (converter);

	return fCharBuffer;
}

